"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HtmlWebpackInjectExternalsPlugin = void 0;
/* eslint-disable import/first */
const fs_1 = require("fs");
const path_1 = require("path");
const chalk_1 = __importDefault(require("chalk"));
const cp_file_1 = __importDefault(require("cp-file"));
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const url_join_1 = __importDefault(require("url-join"));
class HtmlWebpackInjectExternalsPlugin {
    constructor(options) {
        this.name = 'HtmlWebpackInjectExternalsPlugin';
        this.options = options;
    }
    apply(compiler) {
        const generalHost = this.options.host;
        const getBrowserFilePath = (pkg) => {
            const { host, path, fullPath, name } = pkg;
            if (fullPath) {
                return {
                    url: fullPath,
                    attributes: pkg.attributes,
                    injectBefore: pkg.injectBefore,
                    injectAfter: pkg.injectAfter,
                };
            }
            let pkgInfo = {};
            /** mono packages project need to inspact node_modules in more place */
            try {
                // eslint-disable-next-line @typescript-eslint/no-var-requires,global-require,import/no-dynamic-require
                pkgInfo = require(`${name}/package.json`);
            }
            catch (e) {
                const currentPathPkgPath = (0, path_1.join)(process.cwd(), 'node_modules', `${name}/package.json`);
                if ((0, fs_1.existsSync)(currentPathPkgPath)) {
                    // eslint-disable-next-line @typescript-eslint/no-var-requires,global-require,import/no-dynamic-require
                    pkgInfo = require(currentPathPkgPath);
                }
                else {
                    throw new Error(`get package ${name} package.json failed`);
                }
            }
            const { version, unpkg, browser, umd } = pkgInfo;
            const unpkgHost = host || generalHost;
            /**
             * try to guess the browser env usage file path
             * the guess order is reference from unpkg project
             * */
            const browserFilePath = path !== undefined ? path : unpkg || browser || umd || '';
            if (!unpkgHost) {
                throw new Error(`package ${name} missing host`);
            }
            /** 是否为本地模式 */
            const isLocal = Boolean(pkg.local !== undefined ? pkg.local : this.options.local);
            let url = '';
            if (isLocal) {
                const localPrefix = pkg.localPrefix || this.options.localPrefix || '';
                url = (0, path_1.join)('/', localPrefix, `${name}/${browserFilePath}`);
                const packagePath = require.resolve(name).replace(/node_modules.+$/, `/node_modules/${name}`);
                cp_file_1.default.sync((0, path_1.join)(packagePath, browserFilePath), (0, path_1.join)(compiler.options.output.path || 'dist', url));
            }
            else {
                url = fullPath || (0, url_join_1.default)(unpkgHost, `${name}@${version}`, browserFilePath);
            }
            return {
                url,
                version,
                attributes: pkg.attributes,
                injectBefore: pkg.injectBefore,
                injectAfter: pkg.injectAfter,
            };
        };
        const deps = this.options.packages.map(getBrowserFilePath);
        const tags = deps.reduce((r, d) => {
            const result = d.url.endsWith('.css')
                ? {
                    tagName: 'link',
                    voidTag: true,
                    attributes: Object.assign(Object.assign({ rel: 'stylesheet', href: d.url }, this.options.attributes), d.attributes),
                }
                : {
                    tagName: 'script',
                    voidTag: false,
                    attributes: Object.assign(Object.assign({ defer: false, src: d.url }, this.options.attributes), d.attributes),
                };
            if (d.injectBefore) {
                r.push(d.injectBefore);
            }
            r.push(result);
            if (d.injectAfter) {
                r.push(d.injectAfter);
            }
            return r;
        }, []);
        compiler.hooks.compilation.tap(this.name, compilation => {
            html_webpack_plugin_1.default.getHooks(compilation).alterAssetTagGroups.tap(this.name, data => {
                /** remove repeat */
                const toPrependTags = [];
                [...tags].reverse().forEach(tag => {
                    const url = tag.tagName === 'script' ? tag.attributes.src : tag.attributes.href;
                    /**
                     * 检查是否有重复加载的内容
                     * 如果是script则对比src
                     * 如果是link则对比href
                     * */
                    const exist = data.headTags.some(headTag => {
                        if (headTag.tagName === 'script') {
                            return Boolean(headTag.attributes.src) && headTag.attributes.src === url;
                        }
                        if (headTag.tagName === 'link') {
                            return headTag.attributes.href === url;
                        }
                        return false;
                    });
                    if (!exist) {
                        data.headTags.unshift(tag);
                        toPrependTags.push(tag);
                    }
                });
                console.log(chalk_1.default.rgb(73, 204, 144)(`[${this.name}] has prepend these resources in your html head:`));
                toPrependTags.forEach((tag) => {
                    console.log(chalk_1.default.rgb(137, 191, 4)(JSON.stringify(tag)));
                });
                return data;
            });
        });
    }
}
exports.HtmlWebpackInjectExternalsPlugin = HtmlWebpackInjectExternalsPlugin;
